@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document.c 6275 2008-05-10 09:38:26Z pborelli $
+ * $Id: gedit-document.c 6393 2008-08-13 13:34:30Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,7 +38,6 @@
 #include <stdlib.h>
 
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs.h>
 #include <gtksourceview/gtksourceiter.h>
 
 #include "gedit-prefs-manager-app.h"
@@ -108,8 +107,6 @@ struct _GeditDocumentPrivate
 	gchar	    *uri;
 	gint 	     untitled_number;
 
-	GnomeVFSURI *vfs_uri;
-
 	const GeditEncoding *encoding;
 
 	gchar	    *mime_type;
@@ -135,6 +132,10 @@ struct _GeditDocumentPrivate
 	/* Search highlighting support variables */	
 	GeditTextRegion *to_search_region;
 	GtkTextTag      *found_tag;
+	
+	/* Mount operation factory */
+	GeditMountOperationFactory  mount_operation_factory;
+	gpointer		    mount_operation_userdata;
 };
 
 enum {
@@ -277,9 +278,6 @@ gedit_document_finalize (GObject *object)
 	}
 
 	g_free (doc->priv->uri);
-	if (doc->priv->vfs_uri != NULL)
-		gnome_vfs_uri_unref (doc->priv->vfs_uri);
-
 	g_free (doc->priv->mime_type);
 	g_free (doc->priv->search_text);
 
@@ -678,6 +676,7 @@ get_default_style_scheme (void)
 
 	return def_style;
 }
+
 static void
 gedit_document_init (GeditDocument *doc)
 {
@@ -688,7 +687,6 @@ gedit_document_init (GeditDocument *doc)
 	doc->priv = GEDIT_DOCUMENT_GET_PRIVATE (doc);
 
 	doc->priv->uri = NULL;
-	doc->priv->vfs_uri = NULL;
 	doc->priv->untitled_number = get_untitled_number ();
 
 	doc->priv->mime_type = g_strdup ("text/plain");
@@ -760,13 +758,6 @@ set_uri (GeditDocument *doc,
 		g_free (doc->priv->uri);
 		doc->priv->uri = g_strdup (uri);
 
-		if (doc->priv->vfs_uri != NULL)
-			gnome_vfs_uri_unref (doc->priv->vfs_uri);
-
-		/* Note: vfs_uri may be NULL for some valid but
-		 * unsupported uris */
-		doc->priv->vfs_uri = gnome_vfs_uri_new (uri);
-
 		if (doc->priv->untitled_number > 0)
 		{
 			release_untitled_number (doc->priv->untitled_number);
@@ -781,23 +772,21 @@ set_uri (GeditDocument *doc,
 	}
 	else
 	{
-		gchar *base_name = NULL;
-
-		/* Guess the mime type from file extension or fallback to "text/plain" */
-		if (doc->priv->vfs_uri != NULL)
-			base_name = gnome_vfs_uri_extract_short_path_name (doc->priv->vfs_uri);
-		if (base_name != NULL)
+		if (doc->priv->uri != NULL)
 		{
-			const gchar *detected_mime;
+			gchar *content_type;
+			gchar *detected_mime;
 
-			detected_mime = gnome_vfs_get_mime_type_for_name (base_name);
-			if (detected_mime == NULL ||
-			    strcmp (GNOME_VFS_MIME_TYPE_UNKNOWN, detected_mime) == 0)
-				detected_mime = "text/plain";
+			content_type = g_content_type_guess (doc->priv->uri, NULL, 0, NULL);
 
-			doc->priv->mime_type = g_strdup (detected_mime);
+			if (content_type == NULL || g_content_type_is_unknown (content_type))
+				detected_mime = g_strdup ("text/plain");
+			else
+				detected_mime = g_content_type_get_mime_type (content_type);
 
-			g_free (base_name);
+			doc->priv->mime_type = detected_mime != NULL ? detected_mime : g_strdup ("text/plain");
+
+			g_free (content_type);
 		}
 		else
 		{
@@ -864,7 +853,9 @@ void
 gedit_document_set_uri (GeditDocument *doc,
 			const gchar   *uri)
 {
-	gchar * mime_type = g_strdup (doc->priv->mime_type);
+	gchar *mime_type;
+
+	mime_type = g_strdup (doc->priv->mime_type);
 	set_uri (doc, uri, mime_type);
 	
 	g_free (mime_type);
@@ -879,89 +870,8 @@ gedit_document_get_uri_for_display (GeditDocument *doc)
 	if (doc->priv->uri == NULL)
 		return g_strdup_printf (_("Unsaved Document %d"),
 					doc->priv->untitled_number);
-	else if (doc->priv->vfs_uri == NULL)
-		return gnome_vfs_format_uri_for_display (doc->priv->uri);
 	else
-	{	
-		gchar *name;
-		gchar *uri_for_display;
-		
-		name = gnome_vfs_uri_to_string (doc->priv->vfs_uri, GNOME_VFS_URI_HIDE_PASSWORD);
-		g_return_val_if_fail (name != NULL, gnome_vfs_format_uri_for_display (doc->priv->uri));
-		
-		uri_for_display = gnome_vfs_format_uri_for_display (name);
-		g_free (name);
-		
-		return uri_for_display;
-	}
-}
-
-/* move to gedit-utils? */
-static gchar *
-get_uri_shortname_for_display (GnomeVFSURI *uri)
-{
-	gchar *name;	
-	gboolean  validated;
-
-	validated = FALSE;
-
-	name = gnome_vfs_uri_extract_short_name (uri);
-	
-	if (name == NULL)
-	{
-		name = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_PASSWORD);
-	}
-	else if (g_ascii_strcasecmp (uri->method_string, "file") == 0)
-	{
-		gchar *text_uri;
-		gchar *local_file;
-		text_uri = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_PASSWORD);
-		local_file = gnome_vfs_get_local_path_from_uri (text_uri);
-
-		if (local_file != NULL)
-		{
-			g_free (name);
-			name = g_filename_display_basename (local_file);
-			validated = TRUE;
-		}
-
-		g_free (local_file);
-		g_free (text_uri);
-	} 
-	else if (!gnome_vfs_uri_has_parent (uri)) 
-	{
-		const gchar *method;
-
-		method = uri->method_string;
-
-		if (name == NULL ||
-		    strcmp (name, GNOME_VFS_URI_PATH_STR) == 0) 
-		{
-			g_free (name);
-			name = g_strdup (method);
-		} 
-		/*
-		else 
-		{
-			gchar *tmp;
-
-			tmp = name;
-			name = g_strdup_printf ("%s: %s", method, name);
-			g_free (tmp);
-		}
-		*/
-	}
-
-	if (!validated && !g_utf8_validate (name, -1, NULL)) 
-	{
-		gchar *utf8_name;
-
-		utf8_name = gedit_utils_make_valid_utf8 (name);
-		g_free (name);
-		name = utf8_name;
-	}
-
-	return name;
+		return gedit_utils_uri_for_display (doc->priv->uri);
 }
 
 /* Never returns NULL */
@@ -973,10 +883,8 @@ gedit_document_get_short_name_for_display (GeditDocument *doc)
 	if (doc->priv->uri == NULL)
 		return g_strdup_printf (_("Unsaved Document %d"),
 					doc->priv->untitled_number);	      
-	else if (doc->priv->vfs_uri == NULL)
-		return g_strdup (doc->priv->uri);
 	else
-		return get_uri_shortname_for_display (doc->priv->vfs_uri);
+		return gedit_utils_basename_for_display (doc->priv->uri);
 }
 
 /* Never returns NULL */
@@ -1028,9 +936,8 @@ gedit_document_get_readonly (GeditDocument *doc)
 gboolean
 _gedit_document_check_externally_modified (GeditDocument *doc)
 {
-	GnomeVFSFileInfo *info;
-	GnomeVFSResult result;
-	gboolean res = FALSE;
+	GFile *gfile;
+	GFileInfo *info;
 
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
 
@@ -1039,27 +946,20 @@ _gedit_document_check_externally_modified (GeditDocument *doc)
 		return FALSE;
 	}
 
-	info = gnome_vfs_file_info_new ();
-
-	result = gnome_vfs_get_file_info_uri (doc->priv->vfs_uri,
-					      info,
-					      GNOME_VFS_FILE_INFO_DEFAULT|
-					      GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-	if (result != GNOME_VFS_OK)
+	gfile = g_file_new_for_uri (doc->priv->uri);
+	info = g_file_query_info (gfile, G_FILE_ATTRIBUTE_TIME_MODIFIED, G_FILE_QUERY_INFO_NONE, NULL, NULL);
+	g_object_unref (gfile);
+	
+	if (info == NULL)
 	{
-		gnome_vfs_file_info_unref (info);
-
 		return FALSE;
 	}
 
-	if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MTIME)
-	{
-		res = info->mtime > doc->priv->mtime;
-	}
-
-	gnome_vfs_file_info_unref (info);
-
-	return res;
+	GTimeVal timeval;
+	g_file_info_get_modification_time (info, &timeval);
+	g_object_unref (info);
+	
+	return timeval.tv_sec > doc->priv->mtime;
 }
 
 static void
@@ -1082,9 +982,11 @@ document_loader_loaded (GeditDocumentLoader *loader,
 	if (error == NULL)
 	{
 		GtkTextIter iter;
-		const gchar *mime_type;
+		const gchar *content_type;
+		gchar *mime_type;
 
-		mime_type = gedit_document_loader_get_mime_type (loader);
+		content_type = gedit_document_loader_get_content_type (loader);
+		mime_type = g_content_type_get_mime_type (content_type);
 
 		doc->priv->mtime = gedit_document_loader_get_mtime (loader);
 
@@ -1100,6 +1002,8 @@ document_loader_loaded (GeditDocumentLoader *loader,
 		/* We already set the uri */
 		set_uri (doc, NULL, mime_type);
 
+		g_free (mime_type);
+
 		/* move the cursor at the requested line if any */
 		if (doc->priv->requested_line_pos > 0)
 		{
@@ -1145,7 +1049,7 @@ document_loader_loaded (GeditDocumentLoader *loader,
 
 	/* special case creating a named new doc */
 	else if (doc->priv->create &&
-	         (error->code == GNOME_VFS_ERROR_NOT_FOUND) &&
+	         (error->code == G_IO_ERROR_NOT_FOUND) &&
 	         (gedit_utils_uri_has_file_scheme (doc->priv->uri)))
 	{
 		reset_temp_loading_data (doc);
@@ -1178,8 +1082,8 @@ document_loader_loading (GeditDocumentLoader *loader,
 	}
 	else
 	{
-		GnomeVFSFileSize size;
-		GnomeVFSFileSize read;
+		goffset size;
+		goffset read;
 
 		size = gedit_document_loader_get_file_size (loader);
 		read = gedit_document_loader_get_bytes_read (loader);
@@ -1267,10 +1171,13 @@ document_saver_saving (GeditDocumentSaver *saver,
 		if (error == NULL)
 		{
 			const gchar *uri;
-			const gchar *mime_type;
+			const gchar *content_type;
+			gchar *mime_type;
 
 			uri = gedit_document_saver_get_uri (saver);
-			mime_type = gedit_document_saver_get_mime_type (saver);
+
+			content_type = gedit_document_saver_get_content_type (saver);
+			mime_type = g_content_type_get_mime_type (content_type);
 
 			doc->priv->mtime = gedit_document_saver_get_mtime (saver);
 
@@ -1283,6 +1190,8 @@ document_saver_saving (GeditDocumentSaver *saver,
 
 			set_uri (doc, uri, mime_type);
 
+			g_free (mime_type);
+
 			set_encoding (doc, 
 				      doc->priv->requested_encoding, 
 				      TRUE);
@@ -1299,8 +1208,8 @@ document_saver_saving (GeditDocumentSaver *saver,
 	}
 	else
 	{
-		GnomeVFSFileSize size = 0;
-		GnomeVFSFileSize written = 0;
+		goffset size = 0;
+		goffset written = 0;
 
 		size = gedit_document_saver_get_file_size (saver);
 		written = gedit_document_saver_get_bytes_written (saver);
@@ -1441,10 +1350,7 @@ gedit_document_get_deleted (GeditDocument *doc)
 {
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
 
-	if (doc->priv->uri == NULL || doc->priv->vfs_uri == NULL)
-		return FALSE;
-		
-	return !gnome_vfs_uri_exists (doc->priv->vfs_uri);
+	return doc->priv->uri && !gedit_utils_uri_exists (doc->priv->uri);
 }
 
 /*
@@ -2288,3 +2194,26 @@ gedit_document_get_enable_search_highlighting (GeditDocument *doc)
 	
 	return (doc->priv->to_search_region != NULL);
 }
+
+void
+_gedit_document_set_mount_operation_factory (GeditDocument 	       *doc,
+					    GeditMountOperationFactory	callback,
+					    gpointer	                userdata)
+{
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+	
+	doc->priv->mount_operation_factory = callback;
+	doc->priv->mount_operation_userdata = userdata;
+}
+
+GMountOperation *
+_gedit_document_create_mount_operation (GeditDocument *doc)
+{
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), NULL);
+	
+	if (doc->priv->mount_operation_factory == NULL)
+		return g_mount_operation_new ();
+	else
+		return doc->priv->mount_operation_factory (doc, 
+						           doc->priv->mount_operation_userdata);
+}
