@@ -27,12 +27,13 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document.h 6111 2008-01-23 08:26:18Z sfre $
+ * $Id: gedit-document.h 6393 2008-08-13 13:34:30Z pborelli $
  */
  
 #ifndef __GEDIT_DOCUMENT_H__
 #define __GEDIT_DOCUMENT_H__
 
+#include <gio/gio.h>
 #include <gtk/gtk.h>
 #include <gtksourceview/gtksourcebuffer.h>
 #include <libgnomevfs/gnome-vfs.h>
@@ -109,8 +110,8 @@ struct _GeditDocumentClass
 					 gboolean             create);
 
 	void (* loading)		(GeditDocument    *document,
-					 GnomeVFSFileSize  size,
-					 GnomeVFSFileSize  total_size);
+					 goffset	   size,
+					 goffset	   total_size);
 
 	void (* loaded)			(GeditDocument    *document,
 					 const GError     *error);
@@ -122,8 +123,8 @@ struct _GeditDocumentClass
 					 GeditDocumentSaveFlags  flags);
 
 	void (* saving)			(GeditDocument    *document,
-					 GnomeVFSFileSize  size,
-					 GnomeVFSFileSize  total_size);
+					 goffset	   size,
+					 goffset	   total_size);
 
 	void (* saved)  		(GeditDocument    *document,
 					 const GError     *error);
@@ -139,11 +140,10 @@ struct _GeditDocumentClass
 
 enum
 {
-	/* start at GNOME_VFS_NUM_ERRORS since we use GnomeVFSResult 
-	 * for the error codes */ 
-	GEDIT_DOCUMENT_ERROR_EXTERNALLY_MODIFIED = GNOME_VFS_NUM_ERRORS,
-	GEDIT_DOCUMENT_ERROR_NOT_REGULAR_FILE,
+	/* we use GIO error codes, so start at some high number? */
+	GEDIT_DOCUMENT_ERROR_EXTERNALLY_MODIFIED = 1 << 10,
 	GEDIT_DOCUMENT_ERROR_CANT_CREATE_BACKUP,
+	GEDIT_DOCUMENT_ERROR_TOO_BIG,
 	GEDIT_DOCUMENT_NUM_ERRORS 
 };
 
@@ -153,7 +153,11 @@ GType		 gedit_document_get_type      	(void) G_GNUC_CONST;
 
 GeditDocument   *gedit_document_new 		(void);
 
+GFile		*gedit_document_get_location	(GeditDocument       *doc);
+
 gchar		*gedit_document_get_uri 	(GeditDocument       *doc);
+void		 gedit_document_set_uri		(GeditDocument       *doc,
+						 const gchar 	     *uri);
 
 gchar		*gedit_document_get_uri_for_display
 						(GeditDocument       *doc);
@@ -237,6 +241,7 @@ void		 gedit_document_set_enable_search_highlighting
 gboolean	 gedit_document_get_enable_search_highlighting
 						(GeditDocument       *doc);
 
+		
 /* 
  * Non exported functions
  */
@@ -264,6 +269,17 @@ void		_gedit_document_search_region   (GeditDocument       *doc,
 #define GEDIT_SEARCH_SET_CASE_SENSITIVE(sflags,state) ((state == TRUE) ? \
 (sflags |= GEDIT_SEARCH_CASE_SENSITIVE) : (sflags &= ~GEDIT_SEARCH_CASE_SENSITIVE))
 
+typedef GMountOperation *(*GeditMountOperationFactory)(GeditDocument *doc, 
+						       gpointer       userdata);
+
+void		 _gedit_document_set_mount_operation_factory
+						(GeditDocument	            *doc,
+						 GeditMountOperationFactory  callback,
+						 gpointer	             userdata);
+GMountOperation
+		*_gedit_document_create_mount_operation
+						(GeditDocument	     *doc);
+
 G_END_DECLS
 
 #endif /* __GEDIT_DOCUMENT_H__ */
