@@ -51,11 +51,13 @@
 				 G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN "," \
 			 	 G_FILE_ATTRIBUTE_STANDARD_IS_BACKUP "," \
 				 G_FILE_ATTRIBUTE_STANDARD_NAME "," \
-				 G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE
+				 G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE "," \
+				 G_FILE_ATTRIBUTE_STANDARD_ICON
 
 typedef struct _FileBrowserNode    FileBrowserNode;
 typedef struct _FileBrowserNodeDir FileBrowserNodeDir;
 typedef struct _AsyncData	   AsyncData;
+typedef struct _AsyncNode	   AsyncNode;
 
 typedef gint (*SortFunc) (FileBrowserNode * node1,
 			  FileBrowserNode * node2);
@@ -70,6 +72,12 @@ struct _AsyncData
 	gboolean removed;
 };
 
+struct _AsyncNode
+{
+	FileBrowserNodeDir *dir;
+	GCancellable *cancellable;
+};
+
 struct _FileBrowserNode 
 {
 	GFile *file;
@@ -81,6 +89,7 @@ struct _FileBrowserNode
 
 	FileBrowserNode *parent;
 	gint pos;
+	gboolean inserted;
 };
 
 struct _FileBrowserNodeDir 
@@ -150,6 +159,9 @@ static gboolean gedit_file_browser_store_iter_nth_child     (GtkTreeModel * tree
 static gboolean gedit_file_browser_store_iter_parent        (GtkTreeModel * tree_model,
 							     GtkTreeIter * iter,
 							     GtkTreeIter * child);
+static void gedit_file_browser_store_row_inserted	    (GtkTreeModel * tree_model,
+							     GtkTreePath * path,
+							     GtkTreeIter * iter);
 
 static void gedit_file_browser_store_drag_source_init       (GtkTreeDragSourceIface * iface);
 static gboolean gedit_file_browser_store_row_draggable      (GtkTreeDragSource * drag_source,
@@ -169,19 +181,10 @@ static void model_clear                                     (GeditFileBrowserSto
 							     gboolean free_nodes);
 static gint model_sort_default                              (FileBrowserNode * node1,
 							     FileBrowserNode * node2);
-static void print_tree                                      (GeditFileBrowserStore * model,
-							     FileBrowserNode * parent, 
-							     gchar * prefix);
 static void model_check_dummy                               (GeditFileBrowserStore * model,
 							     FileBrowserNode * node);
 static void next_files_async 				    (GFileEnumerator * enumerator,
-							     FileBrowserNodeDir * dir);
-
-static void on_directory_monitor_event                      (GFileMonitor * handle,
-							     GFile *monitor_file,
-							     GFile * info_file,
-							     guint event_type,
-							     FileBrowserNode * parent);
+							     AsyncNode * async);
 
 GEDIT_PLUGIN_DEFINE_TYPE_WITH_CODE (GeditFileBrowserStore, gedit_file_browser_store,
 			G_TYPE_OBJECT,
@@ -206,6 +209,10 @@ enum
 	END_LOADING,
 	ERROR,
 	NO_TRASH,
+	RENAME,
+	BEGIN_REFRESH,
+	END_REFRESH,
+	UNLOAD,
 	NUM_SIGNALS
 };
 
@@ -355,6 +362,41 @@ gedit_file_browser_store_class_init (GeditFileBrowserStoreClass * klass)
 					   no_trash), g_signal_accumulator_true_handled, NULL,
 			  gedit_file_browser_marshal_BOOL__POINTER,
 			  G_TYPE_BOOLEAN, 1, G_TYPE_POINTER);
+	model_signals[RENAME] =
+	    g_signal_new ("rename",
+			  G_OBJECT_CLASS_TYPE (object_class),
+			  G_SIGNAL_RUN_LAST,
+			  G_STRUCT_OFFSET (GeditFileBrowserStoreClass,
+					   rename), NULL, NULL,
+			  gedit_file_browser_marshal_VOID__STRING_STRING, 
+			  G_TYPE_NONE, 2,
+			  G_TYPE_STRING,
+			  G_TYPE_STRING);
+	model_signals[BEGIN_REFRESH] =
+	    g_signal_new ("begin-refresh",
+	    		  G_OBJECT_CLASS_TYPE (object_class),
+	    		  G_SIGNAL_RUN_LAST,
+	    		  G_STRUCT_OFFSET (GeditFileBrowserStoreClass,
+	    		  		   begin_refresh), NULL, NULL,
+	    		  g_cclosure_marshal_VOID__VOID,
+	    		  G_TYPE_NONE, 0);
+	model_signals[END_REFRESH] =
+	    g_signal_new ("end-refresh",
+	    		  G_OBJECT_CLASS_TYPE (object_class),
+	    		  G_SIGNAL_RUN_LAST,
+	    		  G_STRUCT_OFFSET (GeditFileBrowserStoreClass,
+	    		  		   end_refresh), NULL, NULL,
+	    		  g_cclosure_marshal_VOID__VOID,
+	    		  G_TYPE_NONE, 0);
+	model_signals[UNLOAD] =
+	    g_signal_new ("unload",
+	    		  G_OBJECT_CLASS_TYPE (object_class),
+	    		  G_SIGNAL_RUN_LAST,
+	    		  G_STRUCT_OFFSET (GeditFileBrowserStoreClass,
+	    		  		   unload), NULL, NULL,
+	    		  g_cclosure_marshal_VOID__STRING,
+	    		  G_TYPE_NONE, 1,
+	    		  G_TYPE_STRING);
 
 	g_type_class_add_private (object_class,
 				  sizeof (GeditFileBrowserStorePrivate));
@@ -375,6 +417,7 @@ gedit_file_browser_store_iface_init (GtkTreeModelIface * iface)
 	iface->iter_n_children = gedit_file_browser_store_iter_n_children;
 	iface->iter_nth_child = gedit_file_browser_store_iter_nth_child;
 	iface->iter_parent = gedit_file_browser_store_iter_parent;
+	iface->row_inserted = gedit_file_browser_store_row_inserted;
 }
 
 static void
@@ -443,6 +486,13 @@ model_node_visibility (GeditFileBrowserStore * model,
 	return !NODE_IS_FILTERED (node);
 }
 
+static gboolean
+model_node_inserted (GeditFileBrowserStore * model,
+		     FileBrowserNode * node)
+{
+	return node == model->priv->virtual_root || (model_node_visibility (model, node) && node->inserted);
+}
+
 /* Interface implementation */
 
 static GtkTreeModelFlags
@@ -479,10 +529,11 @@ static gboolean
 gedit_file_browser_store_get_iter (GtkTreeModel * tree_model,
 				   GtkTreeIter * iter, GtkTreePath * path)
 {
-	gint *indices, depth, i;
-	FileBrowserNode *node;
-	GeditFileBrowserStore *model;
-	GSList *item;
+	gint * indices, depth, i;
+	FileBrowserNode * node;
+	FileBrowserNode * child;
+	GeditFileBrowserStore * model;
+	GSList * item;
 	gint num;
 
 	g_assert (GEDIT_IS_FILE_BROWSER_STORE (tree_model));
@@ -502,14 +553,12 @@ gedit_file_browser_store_get_iter (GtkTreeModel * tree_model,
 		if (!NODE_IS_DIR (node))
 			return FALSE;
 
-		for (item = FILE_BROWSER_NODE_DIR (node)->children; item;
-		     item = item->next) {
-			if (model_node_visibility
-			    (model, (FileBrowserNode *) (item->data))) {
+		for (item = FILE_BROWSER_NODE_DIR (node)->children; item; item = item->next) {
+			child = (FileBrowserNode *) (item->data);
+
+			if (model_node_inserted (model, child)) {
 				if (num == indices[i]) {
-					node =
-					    (FileBrowserNode *) (item->
-								 data);
+					node = child;
 					break;
 				}
 
@@ -547,8 +596,7 @@ gedit_file_browser_store_get_path_real (GeditFileBrowserStore * model,
 
 		num = 0;
 
-		for (item = FILE_BROWSER_NODE_DIR (node->parent)->children;
-		     item; item = item->next) {
+		for (item = FILE_BROWSER_NODE_DIR (node->parent)->children; item; item = item->next) {
 			check = (FileBrowserNode *) (item->data);
 
 			if (model_node_visibility (model, check)) {
@@ -580,21 +628,20 @@ gedit_file_browser_store_get_path (GtkTreeModel * tree_model,
 {
 	FileBrowserNode *node;
 
-	g_return_val_if_fail (GEDIT_IS_FILE_BROWSER_STORE (tree_model),
-			      NULL);
+	g_return_val_if_fail (GEDIT_IS_FILE_BROWSER_STORE (tree_model), NULL);
 	g_return_val_if_fail (iter != NULL, NULL);
 	g_return_val_if_fail (iter->user_data != NULL, NULL);
 
 	node = (FileBrowserNode *) (iter->user_data);
 
-	return
-	    gedit_file_browser_store_get_path_real
-	    (GEDIT_FILE_BROWSER_STORE (tree_model), node);
+	return gedit_file_browser_store_get_path_real (GEDIT_FILE_BROWSER_STORE (tree_model), 
+						       node);
 }
 
 static void
 gedit_file_browser_store_get_value (GtkTreeModel * tree_model,
-				    GtkTreeIter * iter, gint column,
+				    GtkTreeIter * iter, 
+				    gint column,
 				    GValue * value)
 {
 	FileBrowserNode *node;
@@ -605,9 +652,7 @@ gedit_file_browser_store_get_value (GtkTreeModel * tree_model,
 
 	node = (FileBrowserNode *) (iter->user_data);
 
-	g_value_init (value,
-		      GEDIT_FILE_BROWSER_STORE (tree_model)->priv->
-		      column_types[column]);
+	g_value_init (value, GEDIT_FILE_BROWSER_STORE (tree_model)->priv->column_types[column]);
 
 	switch (column) {
 	case GEDIT_FILE_BROWSER_STORE_COLUMN_URI:
@@ -632,9 +677,10 @@ static gboolean
 gedit_file_browser_store_iter_next (GtkTreeModel * tree_model,
 				    GtkTreeIter * iter)
 {
-	GeditFileBrowserStore *model;
-	FileBrowserNode *node;
-	GSList *item;
+	GeditFileBrowserStore * model;
+	FileBrowserNode * node;
+	GSList * item;
+	GSList * first;
 
 	g_return_val_if_fail (GEDIT_IS_FILE_BROWSER_STORE (tree_model),
 			      FALSE);
@@ -647,12 +693,10 @@ gedit_file_browser_store_iter_next (GtkTreeModel * tree_model,
 	if (node->parent == NULL)
 		return FALSE;
 
-	for (item =
-	     g_slist_next (g_slist_find
-			   (FILE_BROWSER_NODE_DIR (node->parent)->children,
-			    node)); item; item = item->next) {
-		if (model_node_visibility
-		    (model, (FileBrowserNode *) (item->data))) {
+	first = g_slist_next (g_slist_find (FILE_BROWSER_NODE_DIR (node->parent)->children, node));
+
+	for (item = first; item; item = item->next) {
+		if (model_node_inserted (model, (FileBrowserNode *) (item->data))) {
 			iter->user_data = item->data;
 			return TRUE;
 		}
@@ -666,9 +710,9 @@ gedit_file_browser_store_iter_children (GtkTreeModel * tree_model,
 					GtkTreeIter * iter,
 					GtkTreeIter * parent)
 {
-	FileBrowserNode *node;
-	GeditFileBrowserStore *model;
-	GSList *item;
+	FileBrowserNode * node;
+	GeditFileBrowserStore * model;
+	GSList * item;
 
 	g_return_val_if_fail (GEDIT_IS_FILE_BROWSER_STORE (tree_model),
 			      FALSE);
@@ -688,10 +732,8 @@ gedit_file_browser_store_iter_children (GtkTreeModel * tree_model,
 	if (!NODE_IS_DIR (node))
 		return FALSE;
 
-	for (item = FILE_BROWSER_NODE_DIR (node)->children; item;
-	     item = item->next) {
-		if (model_node_visibility
-		    (model, (FileBrowserNode *) (item->data))) {
+	for (item = FILE_BROWSER_NODE_DIR (node)->children; item; item = item->next) {
+		if (model_node_inserted (model, (FileBrowserNode *) (item->data))) {
 			iter->user_data = item->data;
 			return TRUE;
 		}
@@ -705,15 +747,14 @@ filter_tree_model_iter_has_child_real (GeditFileBrowserStore * model,
 				       FileBrowserNode * node)
 {
 	GSList *item;
-
+	
 	if (!NODE_IS_DIR (node))
 		return FALSE;
 
-	for (item = FILE_BROWSER_NODE_DIR (node)->children; item;
-	     item = item->next)
-		if (model_node_visibility
-		    (model, (FileBrowserNode *) (item->data)))
+	for (item = FILE_BROWSER_NODE_DIR (node)->children; item; item = item->next) {
+		if (model_node_inserted (model, (FileBrowserNode *) (item->data)))
 			return TRUE;
+	}
 
 	return FALSE;
 }
@@ -764,10 +805,8 @@ gedit_file_browser_store_iter_n_children (GtkTreeModel * tree_model,
 	if (!NODE_IS_DIR (node))
 		return 0;
 
-	for (item = FILE_BROWSER_NODE_DIR (node)->children; item;
-	     item = item->next)
-		if (model_node_visibility
-		    (model, (FileBrowserNode *) (item->data)))
+	for (item = FILE_BROWSER_NODE_DIR (node)->children; item; item = item->next)
+		if (model_node_inserted (model, (FileBrowserNode *) (item->data)))
 			++num;
 
 	return num;
@@ -800,8 +839,7 @@ gedit_file_browser_store_iter_nth_child (GtkTreeModel * tree_model,
 
 	for (item = FILE_BROWSER_NODE_DIR (node)->children; item;
 	     item = item->next) {
-		if (model_node_visibility
-		    (model, (FileBrowserNode *) (item->data))) {
+		if (model_node_inserted (model, (FileBrowserNode *) (item->data))) {
 			if (num == n) {
 				iter->user_data = item->data;
 				return TRUE;
@@ -840,6 +878,16 @@ gedit_file_browser_store_iter_parent (GtkTreeModel * tree_model,
 	return TRUE;
 }
 
+static void
+gedit_file_browser_store_row_inserted (GtkTreeModel * tree_model,
+				       GtkTreePath * path,
+				       GtkTreeIter * iter)
+{
+	FileBrowserNode * node = (FileBrowserNode *)(iter->user_data);
+	
+	node->inserted = TRUE;
+}
+
 static gboolean
 gedit_file_browser_store_row_draggable (GtkTreeDragSource * drag_source,
 					GtkTreePath       * path)
@@ -996,77 +1044,6 @@ model_sort_default (FileBrowserNode * node1, FileBrowserNode * node2)
 	return collate_nodes (node1, node2);
 }
 
-static void
-model_resort_children (GeditFileBrowserStore * model,
-		       FileBrowserNode * parent, GtkTreePath * path)
-{
-	FileBrowserNodeDir *dir;
-	FileBrowserNode *node;
-	GSList *item;
-	gint *neworder;
-	gint pos = 0;
-	gboolean visible;
-	GtkTreeIter iter;
-	gboolean free_path = FALSE;
-
-	if (!NODE_IS_DIR (parent))
-		return;
-
-	dir = FILE_BROWSER_NODE_DIR (parent);
-
-	if (!dir->children)
-		return;
-
-	visible = model_node_visibility (model, parent);
-
-	if (visible) {
-		pos = 0;
-
-		if (path == NULL) {
-			path = gtk_tree_path_new ();
-			free_path = TRUE;
-		}
-	}
-
-	gtk_tree_path_up (path);
-
-	for (item = dir->children; item; item = item->next) {
-		node = (FileBrowserNode *) (item->data);
-		model_resort_children (model, node, path);
-
-		if (visible && model_node_visibility (model, node)) {
-			node->pos = pos++;
-			gtk_tree_path_next (path);
-		}
-	}
-
-	gtk_tree_path_down (path);
-
-	if (visible) {
-		dir->children =
-		    g_slist_sort (dir->children,
-				  (GCompareFunc) (model->priv->sort_func));
-		neworder = g_new (gint, pos);
-
-		pos = 0;
-
-		for (item = dir->children; item; item = item->next) {
-			node = (FileBrowserNode *) (item->data);
-
-			if (model_node_visibility (model, node))
-				neworder[pos++] = node->pos;
-		}
-
-		iter.user_data = parent;
-		gtk_tree_model_rows_reordered (GTK_TREE_MODEL (model),
-					       path, &iter, neworder);
-		g_free (neworder);
-
-		if (free_path)
-			gtk_tree_path_free (path);
-	}
-}
-
 static void
 model_resort_node (GeditFileBrowserStore * model, FileBrowserNode * node)
 {
@@ -1122,16 +1099,50 @@ model_resort_node (GeditFileBrowserStore * model, FileBrowserNode * node)
 }
 
 static void
-row_inserted (GeditFileBrowserStore * model,
-	      const GtkTreePath * path,
-	      GtkTreeIter * iter)
+row_changed (GeditFileBrowserStore * model,
+	     GtkTreePath ** path,
+	     GtkTreeIter * iter)
 {
-	GtkTreePath *copy = gtk_tree_path_copy (path);
-	
+	GtkTreeRowReference *ref = gtk_tree_row_reference_new (GTK_TREE_MODEL (model), *path);
+
 	/* Insert a copy of the actual path here because the row-inserted
 	   signal may alter the path */
+	gtk_tree_model_row_changed (GTK_TREE_MODEL(model), *path, iter);
+	gtk_tree_path_free (*path);
+	
+	*path = gtk_tree_row_reference_get_path (ref);
+	gtk_tree_row_reference_free (ref);
+}
+
+static void
+row_inserted (GeditFileBrowserStore * model,
+	      GtkTreePath ** path,
+	      GtkTreeIter * iter)
+{
+	/* This function creates a row reference for the path because it's
+	   uncertain what might change the actual model/view when we insert
+	   a node, maybe another directory load is triggered for example. 
+	   Because functions that use this function rely on the notion that
+	   the path remains pointed towards the inserted node, we use the
+	   reference to keep track. */
+	GtkTreeRowReference *ref = gtk_tree_row_reference_new (GTK_TREE_MODEL (model), *path);
+	GtkTreePath * copy = gtk_tree_path_copy (*path);
+
 	gtk_tree_model_row_inserted (GTK_TREE_MODEL(model), copy, iter);
 	gtk_tree_path_free (copy);
+	
+	if (ref) {
+		gtk_tree_path_free (*path);
+
+		/* To restore the path, we get the path from the reference. But, since
+		   we inserted a row, the path will be one index further than the
+		   actual path of our node. We therefore call gtk_tree_path_prev */
+		*path = gtk_tree_row_reference_get_path (ref);
+	
+		gtk_tree_path_prev (*path);
+	}
+
+	gtk_tree_row_reference_free (ref);
 }
 
 static void
@@ -1197,12 +1208,12 @@ model_refilter_node (GeditFileBrowserStore * model,
 		new_visible = model_node_visibility (model, node);
 
 		if (old_visible != new_visible) {
-			iter.user_data = node;
-
 			if (old_visible) {
+				node->inserted = FALSE;
 				row_deleted (model, path);
 			} else {
-				row_inserted (model, path, &iter);
+				iter.user_data = node;
+				row_inserted (model, &path, &iter);
 				gtk_tree_path_next (path);
 			}
 		} else if (old_visible) {
@@ -1299,7 +1310,8 @@ static void
 file_browser_node_free (GeditFileBrowserStore * model,
 			FileBrowserNode * node)
 {
-	FileBrowserNodeDir *dir;
+	FileBrowserNodeDir * dir;
+	gchar * uri;
 
 	if (node == NULL)
 		return;
@@ -1324,9 +1336,15 @@ file_browser_node_free (GeditFileBrowserStore * model,
 		if (dir->hidden_file_hash)
 			g_hash_table_destroy (dir->hidden_file_hash);
 	}
-
+	
 	if (node->file)
+	{
+		uri = g_file_get_uri (node->file);
+		g_signal_emit (model, model_signals[UNLOAD], 0, uri);
+
+		g_free (uri);
 		g_object_unref (node->file);
+	}
 
 	if (node->icon)
 		g_object_unref (node->icon);
@@ -1427,7 +1445,10 @@ model_remove_node (GeditFileBrowserStore * model, FileBrowserNode * node,
 	/* Only delete if the node is visible in the tree (but only when it's
 	   not the virtual root) */
 	if (model_node_visibility (model, node) && node != model->priv->virtual_root)
+	{
+		node->inserted = FALSE;
 		row_deleted (model, path);
+	}
 
 	if (free_path)
 		gtk_tree_path_free (path);
@@ -1485,8 +1506,9 @@ model_clear (GeditFileBrowserStore * model, gboolean free_nodes)
 			if (NODE_IS_DUMMY (dummy)
 			    && model_node_visibility (model, dummy)) {
 				path = gtk_tree_path_new_first ();
-				gtk_tree_model_row_deleted (GTK_TREE_MODEL 
-							    (model), path);
+				
+				dummy->inserted = FALSE;
+				row_deleted (model, path);
 				gtk_tree_path_free (path);
 			}
 		}
@@ -1529,7 +1551,8 @@ file_browser_node_unload (GeditFileBrowserStore * model,
 
 static void
 model_recomposite_icon_real (GeditFileBrowserStore * tree_model,
-			     FileBrowserNode * node)
+			     FileBrowserNode * node,
+			     GFileInfo * info)
 {
 	GtkIconTheme *theme;
 	GdkPixbuf *icon;
@@ -1541,7 +1564,11 @@ model_recomposite_icon_real (GeditFileBrowserStore * tree_model,
 		return;
 
 	theme = gtk_icon_theme_get_default ();
-	icon = gedit_file_browser_utils_pixbuf_from_file (node->file, GTK_ICON_SIZE_MENU);
+	
+	if (info)
+		icon = gedit_file_browser_utils_pixbuf_from_icon (g_file_info_get_icon (info), GTK_ICON_SIZE_MENU);
+	else
+		icon = gedit_file_browser_utils_pixbuf_from_file (node->file, GTK_ICON_SIZE_MENU);
 
 	if (node->icon)
 		g_object_unref (node->icon);
@@ -1584,7 +1611,8 @@ model_recomposite_icon (GeditFileBrowserStore * tree_model,
 
 	model_recomposite_icon_real (tree_model,
 				     (FileBrowserNode *) (iter->
-							  user_data));
+							  user_data),
+				     NULL);
 }
 
 static FileBrowserNode *
@@ -1664,7 +1692,7 @@ model_check_dummy (GeditFileBrowserStore * model, FileBrowserNode * node)
 				    gedit_file_browser_store_get_path_real
 				    (model, dummy);
 
-				row_inserted (model, path, &iter);
+				row_inserted (model, &path, &iter);
 				gtk_tree_path_free (path);
 			}
 		} else {
@@ -1679,9 +1707,9 @@ model_check_dummy (GeditFileBrowserStore * model, FileBrowserNode * node)
 				    (model, dummy);
 				dummy->flags |=
 				    GEDIT_FILE_BROWSER_STORE_FLAG_IS_HIDDEN;
-
-				gtk_tree_model_row_deleted (GTK_TREE_MODEL
-							    (model), path);
+				    
+				dummy->inserted = FALSE;
+				row_deleted (model, path);
 				gtk_tree_path_free (path);
 			}
 		}
@@ -1723,7 +1751,7 @@ model_add_node (GeditFileBrowserStore * model, FileBrowserNode * child,
 		    gedit_file_browser_store_get_path_real (model, child);
 
 		// Emit row inserted
-		row_inserted (model, path, &iter);
+		row_inserted (model, &path, &iter);
 		gtk_tree_path_free (path);
 	}
 
@@ -1758,13 +1786,27 @@ file_browser_node_set_from_info (GeditFileBrowserStore * model,
 	gchar const * name;
 	gboolean free_info = FALSE;
 	GtkTreePath * path;
+	gchar * uri;
+	GError * error = NULL;
 
 	if (info == NULL) {
 		info = g_file_query_info (node->file,  
 					  STANDARD_ATTRIBUTE_TYPES,
 					  G_FILE_QUERY_INFO_NONE,
 					  NULL,
-					  NULL);
+					  &error);
+					  
+		if (!info) {
+			if (!(error->domain == G_IO_ERROR && error->code == G_IO_ERROR_NOT_FOUND)) {
+				uri = g_file_get_uri (node->file);
+				g_warning ("Could not get info for %s: %s", uri, error->message);
+				g_free (uri);
+			}
+			g_error_free (error);
+
+			return;
+		}
+		
 		free_info = TRUE;
 	}
 
@@ -1789,11 +1831,11 @@ file_browser_node_set_from_info (GeditFileBrowserStore * model,
 			node->flags |= GEDIT_FILE_BROWSER_STORE_FLAG_IS_TEXT;		
 	}
 	
+	model_recomposite_icon_real (model, node, info);
+
 	if (free_info)
 		g_object_unref (info);
 
-	model_recomposite_icon_real (model, node);
-
 	if (isadded) {
 		path = gedit_file_browser_store_get_path_real (model, node);
 		model_refilter_node (model, node, path);
@@ -1835,20 +1877,26 @@ model_add_node_from_file (GeditFileBrowserStore * model,
 {
 	FileBrowserNode *node;
 	gboolean free_info = FALSE;
-	
-	if (!info)
-	{
-		info = g_file_query_info (file,
-					  STANDARD_ATTRIBUTE_TYPES,
-					  G_FILE_QUERY_INFO_NONE,
-					  NULL,
-					  NULL);
-		free_info = TRUE;
-	}
-	
+	GError * error = NULL;
+
 	// Check if it already exists
-	if ((node = model_file_exists (model, parent, file)) == NULL) {
-		if (g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY) {
+	if ((node = model_file_exists (model, parent, file)) == NULL) {	
+		if (!info) {
+			info = g_file_query_info (file,
+						  STANDARD_ATTRIBUTE_TYPES,
+						  G_FILE_QUERY_INFO_NONE,
+						  NULL,
+						  &error);
+			free_info = TRUE;
+		}
+		
+		if (!info) {
+			g_warning ("Error querying file info: %s", error->message);
+			g_error_free (error);
+			
+			/* FIXME: What to do now then... */
+			node = file_browser_node_new (file, parent);
+		} else if (g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY) {
 			node = file_browser_node_dir_new (model, file, parent);
 		} else {
 			node = file_browser_node_new (file, parent);
@@ -1856,10 +1904,36 @@ model_add_node_from_file (GeditFileBrowserStore * model,
 
 		file_browser_node_set_from_info (model, node, info, FALSE);
 		model_add_node (model, node, parent);
+		
+		if (info && free_info)
+			g_object_unref (info);
+	}
+
+	return node;
+}
+
+static FileBrowserNode *
+model_add_node_from_dir (GeditFileBrowserStore * model,
+			 FileBrowserNode * parent,
+			 GFile * file)
+{
+	FileBrowserNode *node;
+
+	/* Check if it already exists */
+	if ((node = model_file_exists (model, parent, file)) == NULL) {	
+		node = file_browser_node_dir_new (model, file, parent);
+		file_browser_node_set_from_info (model, node, NULL, FALSE);
+
+		if (node->name == NULL) {
+			file_browser_node_set_name (node);
+		}
+
+		if (node->icon == NULL) {
+			node->icon = gedit_file_browser_utils_pixbuf_from_theme ("folder", GTK_ICON_SIZE_MENU);
+		}
+
+		model_add_node (model, node, parent);
 	}
-	
-	if (free_info)
-		g_object_unref (info);
 
 	return node;
 }
@@ -1936,10 +2010,47 @@ parse_dot_hidden_file (FileBrowserNode *directory)
 	g_free (file_contents);
 }
 
+static void
+on_directory_monitor_event (GFileMonitor * monitor,
+			    GFile * file,
+			    GFile * other_file,
+			    GFileMonitorEvent event_type,
+			    FileBrowserNode * parent)
+{
+	FileBrowserNode *node;
+	FileBrowserNodeDir *dir = FILE_BROWSER_NODE_DIR (parent);
+
+	switch (event_type) {
+	case G_FILE_MONITOR_EVENT_DELETED:
+		node = model_file_exists (dir->model, parent, file);
+
+		if (node != NULL) {
+			// Remove the node
+			model_remove_node (dir->model, node, NULL, TRUE);
+		}
+		break;
+	case G_FILE_MONITOR_EVENT_CREATED:
+		if (g_file_query_exists (file, NULL)) {
+			model_add_node_from_file (dir->model, parent, file, NULL);
+		}
+		
+		break;
+	default:
+		break;
+	}
+}
+
+static void
+async_node_free (AsyncNode *async)
+{
+	g_object_unref (async->cancellable);
+	g_free (async);
+}
+
 static void
 model_iterate_next_files_cb (GFileEnumerator * enumerator, 
 			     GAsyncResult * result, 
-			     FileBrowserNode * parent)
+			     AsyncNode * async)
 {
 	GList * files;
 	GList * item;
@@ -1948,12 +2059,14 @@ model_iterate_next_files_cb (GFileEnumerator * enumerator,
 	GFileType type;
 	gchar const * name;
 	GFile * file;
-	FileBrowserNodeDir * dir = FILE_BROWSER_NODE_DIR (parent);
+	FileBrowserNodeDir * dir = async->dir;
+	FileBrowserNode * parent = (FileBrowserNode *)dir;
 	
 	files = g_file_enumerator_next_files_finish (enumerator, result, &error);
 
 	if (files == NULL) {
 		g_file_enumerator_close (enumerator, NULL, NULL);
+		async_node_free (async);
 		
 		if (!error)
 		{
@@ -1989,6 +2102,10 @@ model_iterate_next_files_cb (GFileEnumerator * enumerator,
 			file_browser_node_unload (dir->model, (FileBrowserNode *)parent, TRUE);
 			g_error_free (error);
 		}
+	} else if (g_cancellable_is_cancelled (async->cancellable)) {
+		/* Check cancel state manually */
+		g_file_enumerator_close (enumerator, NULL, NULL);
+		async_node_free (async);
 	} else {
 		for (item = files; item; item = item->next) {
 			info = G_FILE_INFO (item->data);
@@ -2016,26 +2133,26 @@ model_iterate_next_files_cb (GFileEnumerator * enumerator,
 		}
 		
 		g_list_free (files);
-		next_files_async (enumerator, dir);
+		next_files_async (enumerator, async);
 	}
 }
 
 static void
 next_files_async (GFileEnumerator * enumerator,
-		  FileBrowserNodeDir * dir)
+		  AsyncNode * async)
 {
 	g_file_enumerator_next_files_async (enumerator,
 					    DIRECTORY_LOAD_ITEMS_PER_CALLBACK,
 					    G_PRIORITY_DEFAULT,
-					    dir->cancellable,
+					    async->cancellable,
 					    (GAsyncReadyCallback)model_iterate_next_files_cb,
-					    dir);
+					    async);
 }
 
 static void
 model_iterate_children_cb (GFile * file, 
 			   GAsyncResult * result,
-			   FileBrowserNodeDir * dir)
+			   AsyncNode * async)
 {
 	GError * error = NULL;
 	GFileEnumerator * enumerator;
@@ -2044,6 +2161,9 @@ model_iterate_children_cb (GFile * file,
 	
 	if (enumerator == NULL) {
 		/* Simply return if we were cancelled */
+		FileBrowserNodeDir *dir = async->dir;
+		g_free (async);
+		
 		if (error->domain == G_IO_ERROR && error->code == G_IO_ERROR_CANCELLED)
 			return;
 		
@@ -2056,8 +2176,12 @@ model_iterate_children_cb (GFile * file,
 
 		file_browser_node_unload (dir->model, (FileBrowserNode *)dir, TRUE);
 		g_error_free (error);
+	} else if (g_cancellable_is_cancelled (async->cancellable)) {
+		/* Check cancelled state manually */
+		async_node_free (async);
+		return;
 	} else {
-		next_files_async (enumerator, dir);
+		next_files_async (enumerator, async);
 	}
 }
 
@@ -2083,15 +2207,19 @@ model_load_directory (GeditFileBrowserStore * model,
 	parse_dot_hidden_file (node);
 	
 	dir->cancellable = g_cancellable_new ();
+	
+	AsyncNode *async = g_new (AsyncNode, 1);
+	async->dir = dir;
+	async->cancellable = g_object_ref (dir->cancellable);
 
 	/* Start loading async */
 	g_file_enumerate_children_async (node->file,
 					 STANDARD_ATTRIBUTE_TYPES,
 					 G_FILE_QUERY_INFO_NONE,
 					 G_PRIORITY_DEFAULT,
-					 dir->cancellable,
+					 async->cancellable,
 					 (GAsyncReadyCallback)model_iterate_children_cb,
-					 dir);
+					 async);
 }
 
 static GList *
@@ -2145,8 +2273,7 @@ model_fill (GeditFileBrowserStore * model, FileBrowserNode * node,
 		/* Insert node */
 		iter.user_data = node;
 		
-		row_inserted(model, path, &iter);
-		model_check_dummy (model, node);
+		row_inserted(model, &path, &iter);
 	}
 
 	if (NODE_IS_DIR (node)) {
@@ -2168,45 +2295,13 @@ model_fill (GeditFileBrowserStore * model, FileBrowserNode * node,
 		/* Move back up to node path */
 		gtk_tree_path_up (path);
 	}
+	
+	model_check_dummy (model, node);
 
 	if (free_path)
 		gtk_tree_path_free (path);
 }
 
-static void
-print_tree (GeditFileBrowserStore * model, FileBrowserNode * parent,
-	    gchar * prefix)
-{
-	GSList *item;
-	gchar *newpref;
-
-	if (parent == NULL)
-		parent = model->priv->root;
-
-	if (parent == NULL)
-		return;
-
-	if (prefix == NULL)
-		prefix = "";
-
-	g_message ("%s - %s (%d), loaded: %d", prefix, parent->name,
-		   model_node_visibility (model, parent),
-		   NODE_LOADED (parent));
-
-	if (NODE_IS_DIR (parent)) {
-		newpref = g_strdup_printf ("\t%s", prefix);
-
-		for (item = FILE_BROWSER_NODE_DIR (parent)->children; item;
-		     item = item->next) {
-			print_tree (model,
-				    (FileBrowserNode *) (item->data),
-				    newpref);
-		}
-
-		g_free (newpref);
-	}
-}
-
 static void
 set_virtual_root_from_node (GeditFileBrowserStore * model,
 			    FileBrowserNode * node)
@@ -2308,7 +2403,7 @@ set_virtual_root_from_file (GeditFileBrowserStore * model,
 	for (item = files; item; item = item->next) {
 		check = G_FILE (item->data);
 		
-		parent = model_add_node_from_file (model, parent, check, NULL);
+		parent = model_add_node_from_dir (model, parent, check);
 		g_object_unref (check);
 	}
 
@@ -2398,6 +2493,142 @@ unique_new_name (GFile * directory, gchar const * name)
 	return newuri;
 }
 
+static GeditFileBrowserStoreResult
+model_root_mounted (GeditFileBrowserStore * model, gchar const * virtual_root)
+{
+	model_check_dummy (model, model->priv->root);
+	g_object_notify (G_OBJECT (model), "root");
+
+	if (virtual_root != NULL)
+		return
+		    gedit_file_browser_store_set_virtual_root_from_string
+		    (model, virtual_root);
+	else
+		set_virtual_root_from_node (model,
+					    model->priv->root);
+
+	return GEDIT_FILE_BROWSER_STORE_RESULT_OK;
+}
+
+typedef struct {
+	GeditFileBrowserStore * model;
+	gchar * virtual_root;
+	GMountOperation * operation;
+	GCancellable * cancellable;
+} MountInfo;
+
+
+static void
+handle_root_error (GeditFileBrowserStore * model, GError *error)
+{
+	FileBrowserNode * root;
+
+	g_signal_emit (model, 
+		       model_signals[ERROR], 
+		       0, 
+		       GEDIT_FILE_BROWSER_ERROR_SET_ROOT,
+		       error->message);
+	
+	/* Set the virtual root to the root */
+	root = model->priv->root;
+	model->priv->virtual_root = root;
+	
+	/* Set the root to be loaded */
+	root->flags |= GEDIT_FILE_BROWSER_STORE_FLAG_LOADED;
+	
+	/* Check the dummy */
+	model_check_dummy (model, root);
+	
+	g_object_notify (G_OBJECT (model), "root");
+	g_object_notify (G_OBJECT (model), "virtual-root");
+}
+
+static void
+mount_cb (GFile * file, 
+	  GAsyncResult * res, 
+	  MountInfo * mount_info)
+{
+	gboolean mounted;
+	GError * error = NULL;
+	GeditFileBrowserStore * model = mount_info->model;
+	
+	mounted = g_file_mount_enclosing_volume_finish (file, res, &error);
+	
+	if (g_cancellable_is_cancelled (mount_info->cancellable))
+	{
+		if (error)
+			g_error_free (error);
+		
+		// Reset because it might be reused?
+		g_cancellable_reset (mount_info->cancellable);
+	}
+	else if (mounted)
+	{
+		model_root_mounted (model, mount_info->virtual_root);
+	}
+	else if (error->code != G_IO_ERROR_CANCELLED)
+	{
+		handle_root_error (model, error);
+	}
+	
+	if (error)
+		g_error_free (error);
+
+	g_object_unref (mount_info->operation);
+	g_object_unref (mount_info->cancellable);
+	g_free (mount_info->virtual_root);
+
+	g_free (mount_info);
+}
+
+static GeditFileBrowserStoreResult
+model_mount_root (GeditFileBrowserStore * model, gchar const * virtual_root)
+{
+	GFileInfo * info;
+	GError * error = NULL;
+	MountInfo * mount_info;
+	
+	info = g_file_query_info (model->priv->root->file, 
+				  G_FILE_ATTRIBUTE_STANDARD_TYPE, 
+				  G_FILE_QUERY_INFO_NONE,
+				  NULL,
+				  &error);
+
+	if (!info) {
+		if (error->code == G_IO_ERROR_NOT_MOUNTED) {
+			/* Try to mount it */
+			FILE_BROWSER_NODE_DIR (model->priv->root)->cancellable = g_cancellable_new ();
+			
+			mount_info = g_new(MountInfo, 1);
+			mount_info->model = model;
+			mount_info->virtual_root = g_strdup (virtual_root);
+			
+			/* FIXME: we should be setting the correct window */
+			mount_info->operation = gtk_mount_operation_new (NULL);
+			mount_info->cancellable = g_object_ref (FILE_BROWSER_NODE_DIR (model->priv->root)->cancellable);
+			
+			g_file_mount_enclosing_volume (model->priv->root->file, 
+						       G_MOUNT_MOUNT_NONE,
+						       mount_info->operation,
+						       mount_info->cancellable,
+						       (GAsyncReadyCallback)mount_cb,
+						       mount_info);
+		}
+		else
+		{
+			handle_root_error (model, error);
+		}
+		
+		g_error_free (error);
+	} else {
+		g_object_unref (info);
+		
+		return model_root_mounted (model, virtual_root);
+	}
+	
+	return GEDIT_FILE_BROWSER_STORE_RESULT_OK;
+}
+
 /* Public */
 GeditFileBrowserStore *
 gedit_file_browser_store_new (gchar const *root)
@@ -2445,11 +2676,9 @@ gedit_file_browser_store_set_value (GeditFileBrowserStore * tree_model,
 	model_recomposite_icon (tree_model, iter);
 
 	if (model_node_visibility (tree_model, node)) {
-		path =
-		    gedit_file_browser_store_get_path (GTK_TREE_MODEL
-						       (tree_model), iter);
-		gtk_tree_model_row_changed (GTK_TREE_MODEL (tree_model),
-					    path, iter);
+		path = gedit_file_browser_store_get_path (GTK_TREE_MODEL (tree_model), 
+							  iter);
+		row_changed (tree_model, &path, iter);
 		gtk_tree_path_free (path);
 	}
 }
@@ -2618,13 +2847,6 @@ gedit_file_browser_store_set_root_and_virtual_root (GeditFileBrowserStore *
 
 	if (root != NULL) {
 		file = g_file_new_for_uri (root);
-
-		if (file == NULL) {
-			g_signal_emit (model, model_signals[ERROR], 0,
-				       GEDIT_FILE_BROWSER_ERROR_SET_ROOT,
-				       _("Invalid uri"));
-			return GEDIT_FILE_BROWSER_STORE_RESULT_ERROR;
-		}
 	}
 
 	if (root != NULL && model->priv->root != NULL) {
@@ -2639,8 +2861,7 @@ gedit_file_browser_store_set_root_and_virtual_root (GeditFileBrowserStore *
 	if (virtual_root) {
 		vfile = g_file_new_for_uri (virtual_root);
 
-		if (equal && model->priv->virtual_root &&
-		    g_file_equal (vfile, model->priv->virtual_root->file)) {
+		if (equal && g_file_equal (vfile, model->priv->virtual_root->file)) {
 			if (file)
 				g_object_unref (file);
 
@@ -2661,20 +2882,11 @@ gedit_file_browser_store_set_root_and_virtual_root (GeditFileBrowserStore *
 	if (file != NULL) {
 		/* Create the root node */
 		node = file_browser_node_dir_new (model, file, NULL);
+		
 		g_object_unref (file);
 
 		model->priv->root = node;
-		model_check_dummy (model, node);
-		
-		g_object_notify (G_OBJECT (model), "root");
-
-		if (virtual_root != NULL)
-			return
-			    gedit_file_browser_store_set_virtual_root_from_string
-			    (model, virtual_root);
-		else
-			set_virtual_root_from_node (model,
-						    model->priv->root);
+		return model_mount_root (model, virtual_root);
 	} else {
 		g_object_notify (G_OBJECT (model), "root");
 		g_object_notify (G_OBJECT (model), "virtual-root");
@@ -2816,8 +3028,40 @@ gedit_file_browser_store_refresh (GeditFileBrowserStore * model)
 		return;
 
 	/* Clear the model */
+	g_signal_emit (model, model_signals[BEGIN_REFRESH], 0);
 	file_browser_node_unload (model, model->priv->virtual_root, TRUE);
 	model_load_directory (model, model->priv->virtual_root);
+	g_signal_emit (model, model_signals[END_REFRESH], 0);
+}
+
+static void
+reparent_node (FileBrowserNode * node, gboolean reparent)
+{
+	FileBrowserNodeDir * dir;
+	GSList * child;
+	GFile * parent;
+	gchar * base;
+
+	if (!node->file) {
+		return;
+	}
+	
+	if (reparent) {
+		parent = node->parent->file;
+		base = g_file_get_basename (node->file);
+		g_object_unref (node->file);
+
+		node->file = g_file_get_child (parent, base);
+		g_free (base);
+	}
+	
+	if (NODE_IS_DIR (node)) {
+		dir = FILE_BROWSER_NODE_DIR (node);
+		
+		for (child = dir->children; child; child = child->next) {
+			reparent_node ((FileBrowserNode *)child->data, TRUE);
+		}
+	}
 }
 
 gboolean
@@ -2829,7 +3073,10 @@ gedit_file_browser_store_rename (GeditFileBrowserStore * model,
 	FileBrowserNode *node;
 	GFile * file;
 	GFile * parent;
+	GFile * previous;
 	GError * err = NULL;
+	gchar * olduri;
+	gchar * newuri;
 	GtkTreePath *path;
 
 	g_return_val_if_fail (GEDIT_IS_FILE_BROWSER_STORE (model), FALSE);
@@ -2850,26 +3097,25 @@ gedit_file_browser_store_rename (GeditFileBrowserStore * model,
 	}
 
 	if (g_file_move (node->file, file, G_FILE_COPY_NONE, NULL, NULL, NULL, &err)) {
-		/* TODO: make sure to also re'path all the child nodes that
-		   might be there! */
-		parent = node->file;
+		previous = node->file;
 		node->file = file;
 
 		/* This makes sure the actual info for the node is requeried */
 		file_browser_node_set_name (node);
 		file_browser_node_set_from_info (model, node, NULL, TRUE);
-
-		/* Free the old nodes' GFile */
-		g_object_unref (parent);
 		
+		reparent_node (node, FALSE);
+
 		if (model_node_visibility (model, node)) {
 			path = gedit_file_browser_store_get_path_real (model, node);
-			gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, iter);
+			row_changed (model, &path, iter);
 			gtk_tree_path_free (path);
 
 			/* Reorder this item */
 			model_resort_node (model, node);
 		} else {
+			g_object_unref (previous);
+			
 			if (error != NULL)
 				*error = g_error_new_literal (gedit_file_browser_store_error_quark (),
 							      GEDIT_FILE_BROWSER_ERROR_RENAME,
@@ -2877,6 +3123,15 @@ gedit_file_browser_store_rename (GeditFileBrowserStore * model,
 			return FALSE;
 		}
 
+		olduri = g_file_get_uri (previous);
+		newuri = g_file_get_uri (node->file);
+
+		g_signal_emit (model, model_signals[RENAME], 0, olduri, newuri);
+
+		g_object_unref (previous);
+		g_free (olduri);
+		g_free (newuri);
+
 		return TRUE;
 	} else {
 		g_object_unref (file);
@@ -3182,35 +3437,4 @@ gedit_file_browser_store_new_directory (GeditFileBrowserStore * model,
 	return result;
 }
 
-/* Signal handlers */
-static void
-on_directory_monitor_event (GFileMonitor * monitor,
-			    GFile * file,
-			    GFile * other_file,
-			    GFileMonitorEvent event_type,
-			    FileBrowserNode * parent)
-{
-	FileBrowserNode *node;
-	FileBrowserNodeDir *dir = FILE_BROWSER_NODE_DIR (parent);
-
-	switch (event_type) {
-	case G_FILE_MONITOR_EVENT_DELETED:
-		node = model_file_exists (dir->model, parent, file);
-
-		if (node != NULL) {
-			// Remove the node
-			model_remove_node (dir->model, node, NULL, TRUE);
-		}
-		break;
-	case G_FILE_MONITOR_EVENT_CREATED:
-		if (g_file_query_exists (file, NULL)) {
-			model_add_node_from_file (dir->model, parent, file, NULL);
-		}
-		
-		break;
-	default:
-		break;
-	}
-}
-
 // ex:ts=8:noet:
